/*	Renegade Scripts.dll
	Reborn's Base Defence plugin
	Copyright 2007 Whitedragon(MDB), Jonathan Wilson

	This file is part of the Renegade scripts.dll
	The Renegade scripts.dll is free software; you can redistribute it and/or modify it under
	the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2, or (at your option) any later
	version. See the file COPYING for more details.
	In addition, an exemption is given to allow Run Time Dynamic Linking of this code with any closed source module that does not contain code covered by this licence.
	Only the source code to the module(s) containing the licenced code has to be released.
*/


#include "scripts.h"
#include <stdarg.h>
#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#endif
#include "engine.h"
#include "gmmain.h"
#include "rebbd.h"


BDSettingsStruct *BDSettings = 0;
int ObjectHookID = 0;
ObjectCreateHookStruct *ObjectHookStruct = 0;


int GDIGT1Pool = 0;
int GDIGT2Pool = 0;
int NodTurret1Pool = 0;
int NodTurret2Pool = 0;
int NodTurret1DeadID = 0;
int NodTurret2DeadID = 0;


void ObjectHookCall(void *data,GameObject *obj) {
	if (BDSettings->ToggleBD) {
		if (stricmp(Commands->Get_Preset_Name(obj),"Nod_Turret_Destroyed") == 0) {
			if (Commands->Get_Position(obj).X == BDSettings->Nod_Turret_Pos[1].X && Commands->Get_Position(obj).Y == BDSettings->Nod_Turret_Pos[1].Y && Commands->Get_Position(obj).Z == BDSettings->Nod_Turret_Pos[1].Z) {
				NodTurret1DeadID = Commands->Get_ID(obj);
				Commands->Disable_Physical_Collisions(obj);
			}
			else if (Commands->Get_Position(obj).X == BDSettings->Nod_Turret_Pos[2].X && Commands->Get_Position(obj).Y == BDSettings->Nod_Turret_Pos[2].Y && Commands->Get_Position(obj).Z == BDSettings->Nod_Turret_Pos[2].Z) {
				NodTurret2DeadID = Commands->Get_ID(obj);
				Commands->Disable_Physical_Collisions(obj);
			}
		}
	}
}

void BDSettingsStruct::Load() {
	SettingsLoader::Load();

	LoadBool(ToggleBD,"ToggleBD",false);

	LoadVector3(GDI_GT_Pos[1],"GDIGT1");
	LoadVector3(GDI_GT_Pos[2],"GDIGT2");

	LoadVector3(GDI_GT_Rebuy_Pos[1],"GDIGT1Rebuy");
	LoadVector3(GDI_GT_Rebuy_Pos[2],"GDIGT2Rebuy");

	LoadVector3(Nod_Turret_Pos[1],"NodTurret1");
	LoadVector3(Nod_Turret_Pos[2],"NodTurret2");

	LoadVector3(Nod_Turret_Rebuy_Pos[1],"NodTurret1Rebuy");
	LoadVector3(Nod_Turret_Rebuy_Pos[2],"NodTurret2Rebuy");
}


void Plugin_Load() {
	BDSettings = new BDSettingsStruct("turretpositions.dat");

	ObjectHookStruct = new ObjectCreateHookStruct;
	ObjectHookStruct->hook = ObjectHookCall;
	ObjectHookStruct->data = 0;
	ObjectHookID = AddObjectCreateHook(ObjectHookStruct);
}

void Plugin_Unload() {
	delete BDSettings;
	delete ObjectHookStruct;
}


extern "C" {

DLLEXPORT void SSGM_Level_Loaded_Hook() {
	BDSettings->Load();

	if (BDSettings->ToggleBD && BDSettings->GDI_GT_Pos[1].Y != 0.0f) {
		Console_Input("msg Enabling extra base defences for this map");

		GameObject *Game_GDIGT1, *Game_GDIGT2, *Game_NodTurret1, *Game_NodTurret2;

		Game_GDIGT1 = Commands->Create_Object("GDI_Guard_Tower",BDSettings->GDI_GT_Pos[1]);
		Commands->Set_Player_Type(Game_GDIGT1,1);
		Commands->Attach_Script(Game_GDIGT1,"KAK_Rebuy_BD","1,1");
		Game_GDIGT2 = Commands->Create_Object("GDI_Guard_Tower",BDSettings->GDI_GT_Pos[2]);
		Commands->Set_Player_Type(Game_GDIGT2,1);
		Commands->Attach_Script(Game_GDIGT2,"KAK_Rebuy_BD","1,2");
		Commands->Disable_All_Collisions(Game_GDIGT1);
		Commands->Disable_All_Collisions(Game_GDIGT2);

		Game_NodTurret1 = Commands->Create_Object("Nod_Turret_MP_Improved",BDSettings->Nod_Turret_Pos[1]);
		Commands->Set_Player_Type(Game_NodTurret1,0);
		Commands->Attach_Script(Game_NodTurret1,"KAK_Rebuy_BD","0,1");
		Game_NodTurret2 = Commands->Create_Object("Nod_Turret_MP_Improved",BDSettings->Nod_Turret_Pos[2]);
		Commands->Set_Player_Type(Game_NodTurret2,0);
		Commands->Attach_Script(Game_NodTurret2,"KAK_Rebuy_BD","0,2");
		Commands->Disable_All_Collisions(Game_NodTurret1);
		Commands->Disable_All_Collisions(Game_NodTurret2);
	}
}

}

void KAK_Rebuy_BD::Killed(GameObject *obj, GameObject *shooter) {
	if (Get_Int_Parameter("PlayerType") == 1) {
		if (Get_Int_Parameter("BDNum") == 1) {
			GameObject *Rebuy = Commands->Create_Object("POW_Neuro_Link",BDSettings->GDI_GT_Rebuy_Pos[1]);
			Set_Powerup_Always_Allow_Grant(Rebuy,true);
			Commands->Attach_Script(Rebuy,"KAK_Rebuy_Box","1,GDIGT1");
		}
		else if (Get_Int_Parameter("BDNum") == 2) {
			GameObject *Rebuy = Commands->Create_Object("POW_Neuro_Link",BDSettings->GDI_GT_Rebuy_Pos[2]);
			Set_Powerup_Always_Allow_Grant(Rebuy,true);
			Commands->Attach_Script(Rebuy,"KAK_Rebuy_Box","1,GDIGT2");
		}
	}
	else if (Get_Int_Parameter("PlayerType") == 0) {
		if (Get_Int_Parameter("BDNum") == 1) {
			GameObject *Rebuy = Commands->Create_Object("POW_Neuro_Link",BDSettings->Nod_Turret_Rebuy_Pos[1]);
			Set_Powerup_Always_Allow_Grant(Rebuy,true);
			Commands->Attach_Script(Rebuy,"KAK_Rebuy_Box","0,NodTurret1");
		}
		else if (Get_Int_Parameter("BDNum") == 2) {
			GameObject *Rebuy = Commands->Create_Object("POW_Neuro_Link",BDSettings->Nod_Turret_Rebuy_Pos[2]);
			Set_Powerup_Always_Allow_Grant(Rebuy,true);
			Commands->Attach_Script(Rebuy,"KAK_Rebuy_Box","0,NodTurret2");
		}
	}
}

void KAK_Rebuy_Box::Custom(GameObject *obj, int message, int param, GameObject *sender) {
	float PlayerCreds = Commands->Get_Money(sender);

	if (message == 121646156) {
		if (param == 1) {
			Commands->Start_Timer(obj,this,2,1);
		}
		else if (param == 2) {
			Commands->Start_Timer(obj,this,2,2);
		}
		else if (param == 3) {
			Commands->Start_Timer(obj,this,2,3);
		}
		else if (param == 4) {
			Commands->Start_Timer(obj,this,2,4);
		}
	}
	else if (message == 1000000025) {
		if (Commands->Get_Player_Type(sender) == Get_Int_Parameter("PlayerType")) {
			if (PlayerCreds > 0) {
				if (strstr(Get_Parameter("BDName"),"GDIGT1")) {
					if (PlayerCreds < 100) {
						GDIGT1Pool += (int)PlayerCreds;
						Commands->Give_Money(sender,(PlayerCreds * -1),false);


					}
					else {
						if ((GDIGT1Pool + 100) > 2000) {
							int moneytotake = (100 - ((GDIGT1Pool + 100) - 2000));
							GDIGT1Pool += moneytotake;
							Commands->Give_Money(sender,(moneytotake * -1.0f),false);

						}
						else {
							GDIGT1Pool += 100;
							Commands->Give_Money(sender,-100,false);

						}
					}
					char thankyou[256];
					sprintf(thankyou,"ppage %d Thanks for your donation to the GDI base defence fund, the fund is now at %d of 2000",Get_Player_ID(sender),GDIGT1Pool);
					Console_Input(thankyou);


					if (GDIGT1Pool >= 2000) {
						GDIGT1Pool = 0;
						GameObject *Game_GDIGT1 = Commands->Create_Object("GDI_Guard_Tower",BDSettings->GDI_GT_Pos[1]);
						Commands->Disable_All_Collisions(Game_GDIGT1);
						Commands->Set_Player_Type(Game_GDIGT1,1);
						Commands->Attach_Script(Game_GDIGT1,"KAK_Rebuy_BD","1,1");
                        Console_Input("msg GDI just repurchased Guard Tower one");
					}
					else {
						GameObject *tempCrate = Commands->Create_Object("Invisible_Object",Commands->Get_Position(sender));
						Commands->Attach_Script(tempCrate,"KAK_Rebuy_Box","2,0");
						Commands->Send_Custom_Event(obj,tempCrate,121646156,1,0);
					}
				}
				else if (strstr(Get_Parameter("BDName"),"GDIGT2")) {
					if (PlayerCreds < 100) {
						GDIGT2Pool += (int)PlayerCreds;
						Commands->Give_Money(sender,(PlayerCreds * -1),false);

					}
					else {
						if ((GDIGT2Pool + 100) > 2000) {
							int moneytotake = (100 - ((GDIGT2Pool + 100) - 2000));
							GDIGT2Pool += moneytotake;
							Commands->Give_Money(sender,(moneytotake * -1.0f),false);

						}
						else {
							GDIGT2Pool += 100;
							Commands->Give_Money(sender,-100,false);

						}
					}
					Console_Input(StrFormat("ppage %d Thanks for your donation to the GDI base defence fund, the fund is now at %d of 2000",Get_Player_ID(sender),GDIGT2Pool).c_str());

					if (GDIGT2Pool >= 2000) {
						GDIGT2Pool = 0;
						GameObject *Game_GDIGT2 = Commands->Create_Object("GDI_Guard_Tower",BDSettings->GDI_GT_Pos[2]);
						Commands->Disable_All_Collisions(Game_GDIGT2);
						Commands->Set_Player_Type(Game_GDIGT2,1);
						Commands->Attach_Script(Game_GDIGT2,"KAK_Rebuy_BD","1,2");
						Console_Input("msg GDI just repurchased Guard Tower two");
					}
					else {
						GameObject *tempCrate = Commands->Create_Object("Invisible_Object",Commands->Get_Position(sender));
						Commands->Attach_Script(tempCrate,"KAK_Rebuy_Box","2,0");
						Commands->Send_Custom_Event(obj,tempCrate,121646156,2,0);
					}
				}
				else if (strstr(Get_Parameter("BDName"),"NodTurret1")) {
					if (PlayerCreds < 100) {
						NodTurret1Pool += (int)PlayerCreds;
						Commands->Give_Money(sender,(PlayerCreds * -1),false);

					}
					else {
						if ((NodTurret1Pool + 100) > 2000) {
							int moneytotake = (100 - ((NodTurret1Pool + 100) - 2000));
							NodTurret1Pool += moneytotake;
							Commands->Give_Money(sender,(moneytotake * -1.0f),false);

						}
						else {
							NodTurret1Pool += 100;
							Commands->Give_Money(sender,-100,false);

						}
					}
					Console_Input(StrFormat("ppage %d Thanks for your donation to the Nod base defence fund, the fund is now at %d of 2000",Get_Player_ID(sender),NodTurret1Pool).c_str());

					if (NodTurret1Pool >= 2000) {
						NodTurret1Pool = 0;
						Commands->Destroy_Object(Commands->Find_Object(NodTurret1DeadID));
						GameObject *Game_NodTurret1 = Commands->Create_Object("Nod_Turret_MP_Improved",BDSettings->Nod_Turret_Pos[1]);
						Commands->Disable_All_Collisions(Game_NodTurret1);
						Commands->Set_Player_Type(Game_NodTurret1,0);
						Commands->Attach_Script(Game_NodTurret1,"KAK_Rebuy_BD","0,1");
                        Console_Input("msg Nod just repurchased Turret one");
					}
					else {
						GameObject *tempCrate = Commands->Create_Object("Invisible_Object",Commands->Get_Position(sender));
						Commands->Attach_Script(tempCrate,"KAK_Rebuy_Box","2,0");
						Commands->Send_Custom_Event(obj,tempCrate,121646156,3,0);
					}
				}
				else if (strstr(Get_Parameter("BDName"),"NodTurret2")) {
					if (PlayerCreds < 100) {
						NodTurret2Pool += (int)PlayerCreds;
						Commands->Give_Money(sender,(PlayerCreds * -1),false);

					}
					else {
						if ((NodTurret2Pool + 100) > 2000) {
							int moneytotake = (100 - ((NodTurret2Pool + 100) - 2000));
							NodTurret2Pool += moneytotake;
							Commands->Give_Money(sender,(moneytotake * -1.0f),false);

						}
						else {
							NodTurret2Pool += 100;
							Commands->Give_Money(sender,-100,false);

						}
					}
					Console_Input(StrFormat("ppage %d Thanks for your donation to the Nod base defence fund, the fund is now at %d of 2000",Get_Player_ID(sender),NodTurret2Pool).c_str());
					
					if (NodTurret2Pool >= 2000) {
						NodTurret2Pool = 0;
						Commands->Destroy_Object(Commands->Find_Object(NodTurret2DeadID));
						GameObject *Game_NodTurret2 = Commands->Create_Object("Nod_Turret_MP_Improved",BDSettings->Nod_Turret_Pos[2]);
						Commands->Disable_All_Collisions(Game_NodTurret2);
						Commands->Set_Player_Type(Game_NodTurret2,0);
						Commands->Attach_Script(Game_NodTurret2,"KAK_Rebuy_BD","0,2");
                        Console_Input("msg Nod just repurchased Turret two");
					}
					else {
						GameObject *tempCrate = Commands->Create_Object("Invisible_Object",Commands->Get_Position(sender));
						Commands->Attach_Script(tempCrate,"KAK_Rebuy_Box","2,0");
						Commands->Send_Custom_Event(obj,tempCrate,121646156,4,0);
					}
				}
			}
		}
		else {
			if (strstr(Get_Parameter("BDName"),"GDIGT1")) {
				GameObject *Rebuy = Commands->Create_Object("POW_Neuro_Link",BDSettings->GDI_GT_Rebuy_Pos[1]);
				Set_Powerup_Always_Allow_Grant(Rebuy,true);
				Commands->Attach_Script(Rebuy,"KAK_Rebuy_Box","1,GDIGT1");
			}
			else if (strstr(Get_Parameter("BDName"),"GDIGT2")) {
				GameObject *Rebuy = Commands->Create_Object("POW_Neuro_Link",BDSettings->GDI_GT_Rebuy_Pos[2]);
				Set_Powerup_Always_Allow_Grant(Rebuy,true);
				Commands->Attach_Script(Rebuy,"KAK_Rebuy_Box","1,GDIGT2");
			}
			else if (strstr(Get_Parameter("BDName"),"NodTurret1")) {
				GameObject *Rebuy = Commands->Create_Object("POW_Neuro_Link",BDSettings->Nod_Turret_Rebuy_Pos[1]);
				Set_Powerup_Always_Allow_Grant(Rebuy,true);
				Commands->Attach_Script(Rebuy,"KAK_Rebuy_Box","0,NodTurret1");
			}
			else if (strstr(Get_Parameter("BDName"),"NodTurret2")) {
				GameObject *Rebuy = Commands->Create_Object("POW_Neuro_Link",BDSettings->Nod_Turret_Rebuy_Pos[2]);
				Set_Powerup_Always_Allow_Grant(Rebuy,true);
				Commands->Attach_Script(Rebuy,"KAK_Rebuy_Box","0,NodTurret2");
			}
		}
	}
}

void KAK_Rebuy_Box::Timer_Expired(GameObject *obj, int number) {
	if (number == 1) {
		GameObject *Rebuy = Commands->Create_Object("POW_Neuro_Link",BDSettings->GDI_GT_Rebuy_Pos[1]);
		Set_Powerup_Always_Allow_Grant(Rebuy,true);
		Commands->Attach_Script(Rebuy,"KAK_Rebuy_Box","1,GDIGT1");
	}
	else if (number == 2) {
		GameObject *Rebuy = Commands->Create_Object("POW_Neuro_Link",BDSettings->GDI_GT_Rebuy_Pos[2]);
		Set_Powerup_Always_Allow_Grant(Rebuy,true);
		Commands->Attach_Script(Rebuy,"KAK_Rebuy_Box","1,GDIGT2");
	}
	else if (number == 3) {
		GameObject *Rebuy = Commands->Create_Object("POW_Neuro_Link",BDSettings->Nod_Turret_Rebuy_Pos[1]);
		Set_Powerup_Always_Allow_Grant(Rebuy,true);
		Commands->Attach_Script(Rebuy,"KAK_Rebuy_Box","0,NodTurret1");
	}
	else if (number == 4) {
		GameObject *Rebuy = Commands->Create_Object("POW_Neuro_Link",BDSettings->Nod_Turret_Rebuy_Pos[2]);
		Set_Powerup_Always_Allow_Grant(Rebuy,true);
		Commands->Attach_Script(Rebuy,"KAK_Rebuy_Box","0,NodTurret2");
	}
	Commands->Destroy_Object(obj);
}

ScriptRegistrant<KAK_Rebuy_BD> KAK_Rebuy_BD_Registrant("KAK_Rebuy_BD","PlayerType:int,BDNum:int");
ScriptRegistrant<KAK_Rebuy_Box> KAK_Rebuy_Box_Registrant("KAK_Rebuy_Box","PlayerType:int,BDName:string");

